@@ -88,9 +88,15 @@ convert_to_dwca <- function(path,
   # Write meta.xml ------------------------------------------------------------
   
   if (core_name == "event") {
-    file.copy(
-      from = system.file("extdata", "/dwca_event_core/meta.xml", package = "ecocomDP"),
-      to = path)
+    meta <- xml2::read_xml(
+      system.file(
+        "extdata", "/dwca_event_core/meta.xml", 
+        package = "ecocomDP"
+      )
+    )
+    meta_file <- paste0(derived_id, ".xml")
+    xml2::xml_set_attr(x = meta, attr = "metadata", value = meta_file)
+    xml2::write_xml(meta, file = paste0(path, "/meta.xml"))
   } else if (core_name == "occurrence") {
     # file.copy(
     #   from = system.file("/dwca_occurrence_core/meta.xml", package = "ecocomDP"),
@@ -163,23 +169,19 @@ create_tables_dwca_event_core <- function(
   # Create computed vectors:
   
   # Define new cols specifically needed for DwC-A
+  obs_loc_tax$id <- seq_along(obs_loc_tax$observation_id)
   obs_loc_tax$lsid <- NA_character_
-  obs_loc_tax$dc_occurrence_id <- NA_character_
-  obs_loc_tax$dc_samplingprotocol <-NA_character_
-  obs_loc_tax$dc_event_id <-NA_character_
-  obs_loc_tax$dc_dataset_name <-NA_character_
-  
-  # comb_id 
   obs_loc_tax$dc_occurrence_id <- paste(
-    "occ",
-    derived_id,
-    seq(nrow(obs_loc_tax)),
-    sep = '.')
+    obs_loc_tax$id, 
+    obs_loc_tax$taxon_id,
+    obs_loc_tax$observation_id,
+    sep = "_"
+  )
+  obs_loc_tax$dc_samplingprotocol <- NA_character_
+  obs_loc_tax$dc_event_id <- obs_loc_tax$id
+  obs_loc_tax$dc_dataset_name <- NA_character_
   
-  obs_loc_tax$dc_event_id <- paste(
-    derived_id,
-    obs_loc_tax$event_id,
-    sep = '.')
+  # comb_id
   
   # Use the DOI of the L1 and construct as: "See methods in DOI"
   obs_loc_tax$dc_samplingprotocol <- paste0(
@@ -243,7 +245,7 @@ create_tables_dwca_event_core <- function(
   # Create event --------------------------------------------------------------
   
   event_table <- data.frame(
-    id = seq(nrow(obs_loc_tax)),
+    id = obs_loc_tax$id,
     eventID = obs_loc_tax$dc_event_id,
     datasetName = dc_dataset_name,
     samplingProtocol = obs_loc_tax$dc_samplingprotocol,
@@ -260,8 +262,7 @@ create_tables_dwca_event_core <- function(
   # Create occurrence ---------------------------------------------------------
   
   occurrence_table <- data.frame(
-    id = seq(nrow(obs_loc_tax)),
-    eventID = obs_loc_tax$dc_event_id,
+    id = obs_loc_tax$id,
     occurrenceID = obs_loc_tax$dc_occurrence_id,
     basisOfRecord = obs_loc_tax$dc_basisofrecord,
     scientificName = obs_loc_tax$taxon_name,
@@ -274,15 +275,14 @@ create_tables_dwca_event_core <- function(
   # redundancy
   occurrence_table <- dplyr::distinct_at(
     occurrence_table, 
-    .vars = c("eventID", "basisOfRecord", "scientificName", "taxonID",
+    .vars = c("basisOfRecord", "scientificName", "taxonID",
               "nameAccordingTo", "scientificNameID"),
     .keep_all = TRUE)
   
   # Create extendedmeasurementorfact ------------------------------------------
   
   extendedmeasurementorfact_table <- data.frame(
-    id = seq(nrow(obs_loc_tax)),
-    eventID = obs_loc_tax$dc_event_id,
+    id = obs_loc_tax$id,
     occurrenceID = obs_loc_tax$dc_occurrence_id,
     measurementType = obs_loc_tax$variable_name,
     measurementTypeID = NA_character_,
@@ -389,12 +389,13 @@ make_eml_dwca <- function(path,
   # Validate inputs -----------------------------------------------------------
   
   # The parent data package should exist
-  missing_parent_data_package <- suppressWarnings(
-    stringr::str_detect(
-      suppressMessages(
-        api_read_metadata(source_id, environment)), 
-      "Unable to access metadata for packageId:"))
-  if (missing_parent_data_package) {
+  parent_data_package_eml <- suppressMessages(
+    api_read_metadata(source_id, environment)
+  )
+  if (typeof(parent_data_package_eml) == "character") {
+    # A type of character indicates there was a failure in reading the parent 
+    # data package EML (we are expecting a list of XML elements), and therefore 
+    # cannot proceed with processing.
     stop(
       "The L1 data package '", source_id, "' does not exist.",
       call. = FALSE)
@@ -639,26 +640,27 @@ make_eml_dwca <- function(path,
   
   message("    <abstract>")
   
-  # Add links to L0 and L1 data packages
-  eml_L2$dataset$abstract$para[[1]] <- stringr::str_replace(
-    eml_L2$dataset$abstract$para[[1]], 
-    "L0_PACKAGE_URL", 
-    url_grandparent)
-  eml_L2$dataset$abstract$para[[1]] <- stringr::str_replace(
-    eml_L2$dataset$abstract$para[[1]], 
-    "L1_PACKAGE_URL", 
-    url_parent)
+  # Constructing the L2 abstract paragraph here because "<ulink>" required by 
+  # GBIF, gets mangled when reading from file with EML::set_TextType()
+  
+  L2_para <- paste0(
+    "This data package is formatted as a Darwin Core Archive (DwC-A, event ",
+    "core). For more information on Darwin Core see ",
+    "<ulink>https://www.tdwg.org/standards/dwc/</ulink>. This Level 2 data ",
+    "package was derived from the Level 1 data package found here: <ulink>",
+    url_parent, "</ulink>, which was derived from the Level 0 data package ",
+    "found here: <ulink>", url_grandparent, "</ulink>. The abstract above was",
+    " extracted from the Level 0 data package and is included for context."
+  )
   
-  # Parse para from xml object because emld parsing is irregular
-  L2_para <- eml_L2$dataset$abstract$para[[1]]
   L0_para <- xml2::xml_text(
     xml2::xml_find_all(xml_L0, ".//abstract//para"))
   eml_L1$dataset$abstract <- NULL
   
   # Create L2 abstract
   eml_L1$dataset$abstract$para <- c(
-    list(L2_para),
-    list(L0_para))
+    list(L0_para),
+    list(L2_para))
   
   # Update <keywordSet> -------------------------------------------------------
   
@@ -681,21 +683,51 @@ make_eml_dwca <- function(path,
   
   # Update <intellectualRights> -----------------------------------------------
   
-  # Use parent intellectual rights or CC0 if none exists
-  if (is.null(eml_L1$dataset$intellectualRights)) {
-    message("    <intellectualRights>")
-    eml_L1$dataset$intellectualRights <- eml_L2$dataset$intellectualRights
-  }
+  # FIXME Are we keeping the original license or adding our own? Any change
+  # here should also be applied to create_eml() for the L1.
+  
+  message("    <intellectualRights>")
+  eml_L1$dataset$intellectualRights <- list(
+    para = paste0(
+      "This data package is released to the public domain under Creative ",
+      "Commons CC0 1.0 No Rights Reserved (see: ",
+      "<ulink>https://creativecommons.org/publicdomain/zero/1.0/</ulink>). ",
+      "It is considered professional etiquette to provide attribution of ",
+      "the original work if this data package is shared in whole or by ",
+      "individual components. A generic citation is provided for this data ",
+      "package on the website ",
+      "<ulink>https://portal.edirepository.org</ulink> (herein website) ",
+      "in the summary metadata page. Communication (and collaboration) with ",
+      "the creators of this data package is recommended to prevent duplicate ",
+      "research or publication. This data package (and its components) is ",
+      "made available as is and with no warranty of accuracy or fitness for ",
+      "use. The creators of this data package and the website shall not be ",
+      "liable for any damages resulting from misinterpretation or misuse of ",
+      "the data package or its components. Periodic updates of this data ",
+      "package may be available from the website. Thank you."
+    )
+  )
   
   # Update <methods> ----------------------------------------------------------
   
   message("    <methods>")
   
-  # Parse components to be reordered and recombined. Parse para from xml 
-  # object because emld parsing is irregular
+  # Constructing the L2 methods paragraph here because "<ulink>" required by 
+  # GBIF, gets mangled when reading from file with EML::set_methods()
   
-  methods_L2 <- eml_L2$dataset$methods$methodStep[1]
-  eml_L2$dataset$methods$methodStep[[1]] <- NULL
+  methods_L2 <- list(
+    description = list(
+      para = paste0(
+        "This data package is a child (Level 2) of the parent data package ",
+        "(Level 1: <ulink>", url_parent, "</ulink>), and listed in the ",
+        "dataSource element below. This Level 2 data package was created ",
+        "using the convert_to_dwca() function of the ecocomDP R library: ",
+        "<ulink>https://github.com/EDIorg/ecocomDP</ulink>. Methods of the ",
+        "Level 0 original source dataset (<ulink>", url_grandparent, 
+        "</ulink>) are included below for context:"
+      )
+    )
+  )
   
   provenance_L1 <- suppressMessages(
     api_get_provenance_metadata(
