@@ -521,6 +521,12 @@ create_eml <- function(path,
   # user_domain here expands editing permission to the creator of the DwC-A 
   # data package this EML will be apart of.
   
+  # A patch for EML::read_eml() handling of nodes with 1 child vs > 1 child
+  if (!is.null(names(eml_L0$access$allow))) {
+    eml_L0$access$allow <- list(eml_L0$access$allow)
+  }
+  
+  
   eml_L0$access$allow <- unique(
     c(eml_L0$access$allow, 
       eml_L1$access$allow))
